"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectUnusedDependencyErrors = exports.collectMissingDependencyErrors = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
const fs = (0, tslib_1.__importStar)(require("fs"));
const path = (0, tslib_1.__importStar)(require("path"));
const module_1 = (0, tslib_1.__importDefault)(require("module"));
function collectMissingDependencyErrors(node, file, packageJsonMap, missingDependencyErrors, sourceFile, ignoreModules) {
    if (typescript_1.default.isImportDeclaration(node)) {
        if (typescript_1.default.isStringLiteral(node.moduleSpecifier)) {
            checkImport(node.moduleSpecifier, file, packageJsonMap, missingDependencyErrors, sourceFile, ignoreModules);
        }
    }
    else if (typescript_1.default.isImportEqualsDeclaration(node)
        && typescript_1.default.isExternalModuleReference(node.moduleReference)
        && typescript_1.default.isStringLiteral(node.moduleReference.expression)) {
        checkImport(node.moduleReference.expression, file, packageJsonMap, missingDependencyErrors, sourceFile, ignoreModules);
    }
}
exports.collectMissingDependencyErrors = collectMissingDependencyErrors;
function collectUnusedDependencyErrors(unusedDependencyErrors, packageJsonMap, needModules) {
    for (const [file, values] of packageJsonMap) {
        const absolutePath = path.relative('.', file).split('\\').join('/');
        for (const value of values) {
            if (!value.imported && !needModules.includes(value.name)) {
                unusedDependencyErrors.push({ file: absolutePath, name: value.name, line: 0, character: 0, type: `'package.json'` });
            }
        }
    }
}
exports.collectUnusedDependencyErrors = collectUnusedDependencyErrors;
function checkImport(stringLiteral, file, packageJsonMap, missingDependencyErrors, sourceFile, ignoreModules) {
    if (stringLiteral.text.startsWith('.')) {
        return;
    }
    const moduleNameParts = stringLiteral.text.split('/');
    const moduleName = moduleNameParts[0].startsWith('@') && moduleNameParts.length > 1
        ? moduleNameParts[0] + '/' + moduleNameParts[1]
        : moduleNameParts[0];
    if (module_1.default.Module.builtinModules.includes(moduleName)) {
        return;
    }
    if (ignoreModules.includes(moduleName)) {
        return;
    }
    const packageJson = getPackageJson(file, packageJsonMap);
    const dependency = packageJson.find(p => p.name === moduleName);
    if (dependency) {
        dependency.imported = true;
    }
    else {
        const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(sourceFile, stringLiteral.getStart(sourceFile));
        missingDependencyErrors.push({ file, name: moduleName, line, character, type: `'import'` });
    }
}
function getPackageJson(file, map) {
    const dirname = path.dirname(file);
    const packageJsonPath = path.resolve(dirname, 'package.json');
    let dependencies = map.get(packageJsonPath);
    if (dependencies) {
        return dependencies;
    }
    let packageJson;
    try {
        packageJson = JSON.parse(fs.readFileSync(packageJsonPath, { encoding: 'utf8' }));
    }
    catch (_a) {
        // no action
    }
    if (packageJson) {
        dependencies = [];
        if (typeof packageJson.dependencies === 'object' && packageJson.dependencies) {
            dependencies.push(...Object.keys(packageJson.dependencies).map(d => ({ name: d, imported: false })));
        }
        if (typeof packageJson.peerDependencies === 'object' && packageJson.peerDependencies) {
            dependencies.push(...Object.keys(packageJson.peerDependencies).map(d => ({ name: d, imported: false })));
        }
        map.set(packageJsonPath, dependencies);
        return dependencies;
    }
    if (dirname === '.') {
        map.set(packageJsonPath, []);
        return [];
    }
    return getPackageJson(dirname, map);
}
