"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.check = void 0;
const tslib_1 = require("tslib");
const typescript_1 = (0, tslib_1.__importDefault)(require("typescript"));
const fs = (0, tslib_1.__importStar)(require("fs"));
const path = (0, tslib_1.__importStar)(require("path"));
const dependency_1 = require("./dependency");
const template_1 = require("./template");
const key_1 = require("./key");
const export_1 = require("./export");
const class_member_1 = require("./class-member");
const promise_1 = require("./promise");
function check(uniqFiles, ignoreModules, needModules, strict) {
    const languageService = getLanguageService(uniqFiles);
    const program = typescript_1.default.createProgram(uniqFiles, { target: typescript_1.default.ScriptTarget.ESNext });
    const checker = program.getTypeChecker();
    const unusedExportsErrors = [];
    const unreferencedMembersErrors = [];
    const canOnlyBePublicErrors = [];
    const missingKeyErrors = [];
    const missingDependencyErrors = [];
    const unusedDependencyErrors = [];
    const promiseNotAwaitErrors = [];
    const packageJsonMap = new Map();
    for (const file of uniqFiles) {
        const sourceFile = program.getSourceFile(file);
        if (sourceFile === undefined) {
            continue;
        }
        sourceFile.forEachChild(node => {
            (0, export_1.collectUnusedExportsErrors)(node, file, sourceFile, languageService, unusedExportsErrors);
            if (typescript_1.default.isClassDeclaration(node)) {
                const referencedMembers = (0, class_member_1.collectReferencedMembers)(node, languageService, file, node);
                const decorators = node.decorators;
                if (decorators) {
                    for (const decorator of decorators) {
                        if (typescript_1.default.isCallExpression(decorator.expression)
                            && typescript_1.default.isIdentifier(decorator.expression.expression)
                            && decorator.expression.expression.text === 'Component'
                            && decorator.expression.arguments.length > 0) {
                            const argument = decorator.expression.arguments[0];
                            if (typescript_1.default.isObjectLiteralExpression(argument)) {
                                for (const property of argument.properties) {
                                    if (typescript_1.default.isPropertyAssignment(property) && typescript_1.default.isIdentifier(property.name)) {
                                        const propertyName = property.name.text;
                                        if (propertyName === 'template') {
                                            const text = getText(program, languageService, file, property.initializer);
                                            (0, template_1.collectCanOnlyBePublicErrors)(node.members, referencedMembers, text, canOnlyBePublicErrors, file, sourceFile, node);
                                            (0, key_1.collectMissingKeyErrors)(propertyName, property.initializer, text, missingKeyErrors, file, sourceFile);
                                        }
                                        else if (propertyName === 'props') {
                                            if (typescript_1.default.isArrayLiteralExpression(property.initializer)) {
                                                for (const member of node.members) {
                                                    if (!referencedMembers.has(member)
                                                        && member.name
                                                        && typescript_1.default.isIdentifier(member.name)) {
                                                        const text = member.name.text;
                                                        if (property.initializer.elements.some(e => getText(program, languageService, file, e) === text)) {
                                                            referencedMembers.add(member);
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        else if (propertyName === 'templateUrl') {
                                            const url = getText(program, languageService, file, property.initializer);
                                            if (url) {
                                                let text;
                                                try {
                                                    text = fs.readFileSync(path.resolve(path.dirname(file), url), { encoding: 'utf8' });
                                                }
                                                catch (_a) {
                                                    // no action
                                                }
                                                (0, template_1.collectCanOnlyBePublicErrors)(node.members, referencedMembers, text, canOnlyBePublicErrors, file, sourceFile, node);
                                                (0, key_1.collectMissingKeyErrors)(propertyName, property.initializer, text, missingKeyErrors, file, sourceFile);
                                            }
                                        }
                                        else if (propertyName === 'host' && typescript_1.default.isObjectLiteralExpression(property.initializer)) {
                                            const hostProperties = property.initializer.properties;
                                            for (const hostProperty of hostProperties) {
                                                if (typescript_1.default.isPropertyAssignment(hostProperty)) {
                                                    const key = getText(program, languageService, file, hostProperty.name);
                                                    if (key && (0, template_1.isAngularAttrName)(key)) {
                                                        const text = getText(program, languageService, file, hostProperty.initializer);
                                                        (0, template_1.collectCanOnlyBePublicErrors)(node.members, referencedMembers, text, canOnlyBePublicErrors, file, sourceFile, node);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                (0, class_member_1.collectUnreferencedMembersErrors)(referencedMembers, unreferencedMembersErrors, file, sourceFile, node);
            }
            if (strict) {
                (0, dependency_1.collectMissingDependencyErrors)(node, file, packageJsonMap, missingDependencyErrors, sourceFile, ignoreModules);
                for (const child of iterateNode(node)) {
                    (0, promise_1.collectPromiseNotAwaitErrors)(child, checker, promiseNotAwaitErrors, file, sourceFile);
                }
            }
        });
    }
    if (strict) {
        (0, dependency_1.collectUnusedDependencyErrors)(unusedDependencyErrors, packageJsonMap, needModules);
    }
    return {
        unusedExportsErrors,
        unreferencedMembersErrors,
        canOnlyBePublicErrors,
        missingKeyErrors,
        missingDependencyErrors,
        unusedDependencyErrors,
        promiseNotAwaitErrors
    };
}
exports.check = check;
function* iterateNode(node) {
    yield node;
    const children = node.getChildren();
    for (const child of children) {
        yield* iterateNode(child);
    }
}
function getText(program, languageService, file, node) {
    if (typescript_1.default.isNoSubstitutionTemplateLiteral(node)
        || typescript_1.default.isStringLiteral(node)) {
        return node.text;
    }
    else if (typescript_1.default.isIdentifier(node)) {
        const definitions = languageService.getDefinitionAtPosition(file, node.end);
        if (definitions && definitions.length > 0) {
            const definition = definitions[0];
            const child = findNodeAtDefinition(program, definition);
            if (child && typescript_1.default.isVariableStatement(child)) {
                return getVariableValue(child, definition.name, program, languageService, file);
            }
        }
    }
    return undefined;
}
function findNodeAtDefinition(program, definition) {
    const sourceFile = program.getSourceFile(definition.fileName);
    if (sourceFile) {
        return sourceFile.forEachChild(child => {
            if (child.pos < definition.textSpan.start && child.end > definition.textSpan.start + definition.textSpan.length) {
                return child;
            }
            return undefined;
        });
    }
    return undefined;
}
function getVariableValue(child, variableName, program, languageService, file) {
    const declarations = child.declarationList.declarations;
    for (const declaration of declarations) {
        if (typescript_1.default.isVariableDeclaration(declaration)) {
            const name = declaration.name;
            if (typescript_1.default.isIdentifier(name)
                && name.text === variableName) {
                const initializer = declaration.initializer;
                if (initializer) {
                    return getText(program, languageService, file, initializer);
                }
            }
        }
    }
    return undefined;
}
function getLanguageService(uniqFiles) {
    return typescript_1.default.createLanguageService({
        getCompilationSettings() {
            return {
                jsx: typescript_1.default.JsxEmit.React
            };
        },
        getScriptFileNames() {
            return uniqFiles;
        },
        getScriptVersion() {
            return '';
        },
        getScriptSnapshot(fileName) {
            if (fileName === '.ts') {
                return typescript_1.default.ScriptSnapshot.fromString('');
            }
            return typescript_1.default.ScriptSnapshot.fromString(fs.readFileSync(fileName, { encoding: 'utf8' }));
        },
        getCurrentDirectory: () => '.',
        getDefaultLibFileName(options) {
            return typescript_1.default.getDefaultLibFilePath(options);
        },
        fileExists: typescript_1.default.sys.fileExists,
        readFile: typescript_1.default.sys.readFile,
        readDirectory: typescript_1.default.sys.readDirectory
    });
}
